/*===============================================================================
Title: dwMegaWifi.ino
Description: deviceWISE Mega with WiFi Sample Program

Copyright (c)2015 Telit.
http://www.ilstechnology.com
http://www.devicewise.com


Legal Disclaimer:

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

    Neither the name of ILS Technology nor Telit nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

===================================================================================*/


#include <SPI.h>
#include <WiFi.h>

#include <dwArduinoWiFiOpenClient.h>
  
//Wi-Fi Network - SSID / Password
#define WIFI_SSID          "UserGUEST"
#define WIFI_PW            "UserPassword"
  

// Authentication/Registration Details      
#define DWOPEN_APPTOKEN    "IgP2lz2ghabxqw7J"      //Application Token
#define DWOPEN_SERVER      "api.devicewise.com"    //deviceWISE server
#define DWOPEN_PORT        1883                    //MQTT Port

//ILST Headquarters - GPS Detail...
#define ILST_Latitude      "26.394919"
#define ILST_Longitude     "-80.112808"
#define ILST_Altitude      "3.0"
#define ILST_Speed         "0.0"
#define ILST_Heading       "0.0"

char dwThingKey[40];
byte mac[6] = {0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5};

void callback(char* topic, uint8_t* payload, unsigned int length);

WiFiClient c;


#define RED_LED           4               //D4 - RED LED
#define BUZZER_PIN        2               //D2 - Buzzer
#define ANGLE_PIN         A0              //A0 - Angle/Rotary Sensor  


dwOpenClient dwClient((char*) DWOPEN_SERVER, DWOPEN_PORT, callback, &c);

void setup() 
{
  int i;
  
  Serial.begin(9600);
  while( !Serial ) delay( 100 );  
  
  for(i = 5; i > 0; i-- )
  {
    Serial.print("Ready to Execute... Countdown = ");
    Serial.println(i); 
    delay( 1000 );
  }
 

  Serial.print("Attempting to connect to Network named: "); Serial.println(WIFI_SSID); 
  WiFi.begin(WIFI_SSID, WIFI_PW);
  
  while( WiFi.status() != WL_CONNECTED ) 
  {
    Serial.print(".");
    delay(5000);
    WiFi.begin(WIFI_SSID, WIFI_PW);
    Serial.print("WiFi Status = "); Serial.println(WiFi.status());
  }
  
  Serial.println("\nYou're connected to the network\n\rWaiting for an ip address");
  
  while (WiFi.localIP() == INADDR_NONE) 
  {
    Serial.print(".");
    delay(300);
  }

  Serial.print("\nIP Address obtained : "); Serial.println(WiFi.localIP());
  
  //Get my MAC address
  WiFi.macAddress(mac);

  //Format the ThingKey for this run using our MAC
  sprintf(dwThingKey, "ArduinoMega-%x%x%x%x%x%x", mac[5], mac[4], mac[3], mac[2], mac[1], mac[0]);  
  Serial.print("\nMy Thing Key: ");  Serial.println( dwThingKey );   
  
  //Configure RED LEDs
  pinMode(RED_LED, OUTPUT);
  digitalWrite(RED_LED, LOW);  

  //Configure Buzzer
  pinMode(BUZZER_PIN, OUTPUT);
  digitalWrite(BUZZER_PIN, LOW);
  
}

void loop() 
{
  int rc = -1;
  char szText[128];
  char szPropValue[16];
  static int LastRotary = -1;

  //Already connected to the cloud?
  if( !dwClient.connected() )
  {
      //Not connected, try connecting to the cloud.
      
      Serial.print("\nConnecting to: "); Serial.print(DWOPEN_SERVER); Serial.print(" with client id: "); Serial.println(dwThingKey);
  
      dwClient.connect((char*) dwThingKey, (char*) dwThingKey, (char*) DWOPEN_APPTOKEN);

      //Did we connect to the cloud?
      if( dwClient.connected() )
      {
          //Yes, we're connected to the cloud.
          Serial.println("Connected to m2mAIR Cloud...");      
          dwClient.dwSleep(1000);   
                
          //Serial.println("Subscribing to Topic 'reply'...");
          dwClient.subscribe(MQTT_REPLY);
          dwClient.dwSleep(1000);       
                
          //Serial.println("Subscribing to Topic 'notify'...");
          dwClient.subscribe(MQTT_NOTIFY);
          dwClient.dwSleep(1000);      
           
          //Publish Log Message... Connected
          Serial.println("\ndwLaunchPad- Log Publish In Process..." );
          rc = dwClient.dwLogPublish( (char*) dwThingKey, (char*) "Arduino Mega IoT Application Started...");      
          Serial.print("Publish Log Message return code : "); Serial.println(rc);       
          dwClient.dwSleep(1000);      
    
     
          //Publish Location Message...
          Serial.println("\ndwOpen API - Location Publish In Process..." );      
          rc = dwClient.dwLocationPublish( dwThingKey, ILST_Latitude, ILST_Longitude, ILST_Altitude, ILST_Speed, ILST_Heading );            
          Serial.print("Publish Location return code : "); Serial.println(rc);          
          dwClient.dwSleep(1000);      
          
          //Publish Attribute...
          Serial.println("\ndwOpen API - Attribute Publish In Process..." );    
          sprintf(szPropValue, "%x%x%x%x%x%x", mac[5], mac[4], mac[3], mac[2], mac[1], mac[0]);     
          rc = dwClient.dwAttributePublish( dwThingKey, "MAC", szPropValue );    
          
          Serial.print("Publish Location return code : "); Serial.println(rc);    
          dwClient.dwSleep(1000);            
       
      }    
       
  }


  //Are we connected to the cloud?
  if( dwClient.connected() )   
  {  
        //Yes, Read sensor value from potentiometer
        int RotaryValue = analogRead(ANGLE_PIN); 
  
        //Did the Rotary Value change?
        if (RotaryValue != LastRotary) 
        {
            //Yes, Publish new rotary value
            Serial.print(dwThingKey); Serial.print(", RotaryValue:"); Serial.println(RotaryValue);         
            sprintf(szPropValue,"%d", RotaryValue);             
            
            Serial.print(dwThingKey); Serial.println(", dwOpen API - Executing Rotary Property Publish...");
            rc = dwClient.dwPropertyPublish( dwThingKey, "Rotary", szPropValue );     
            Serial.print(dwThingKey); Serial.print(",dwOpen API - Rotary Property Publish Returned... rc = "); Serial.println(rc);
            
            //------------------------------------------------
            //-- Alarm Publish
            //------------------------------------------------
  
            int alarmState;
            char alarmMsg[40];
            
            // Various simulated alarm status...
            if (RotaryValue < 200)
            {
              alarmState = 0;
              strcpy( alarmMsg, "Temp - OK" );
              Serial.print(dwThingKey); Serial.println(", dwOpen API - Executing Alarm Publish Request... OK");    
            }
            else if (RotaryValue < 900)
            {
                alarmState = 1;
                strcpy( alarmMsg, "Temp - Warning");
                Serial.print(dwThingKey); Serial.println(", dwOpen API - Executing Alarm Publish Request... Warning");
            }
            else
            {
                alarmState = 2;
                strcpy(alarmMsg, "Temp - HIGH" );
                Serial.print(dwThingKey); Serial.println(", dwOpen API - Executing Alarm Publish Request... HIGH");
            }
                  
            rc = dwClient.dwAlarmPublish( dwThingKey, "alarm1", alarmState, alarmMsg );
            Serial.print(dwThingKey); Serial.print(",dwOpen API - Alarm Publish Returned... rc = "); Serial.println(rc);            
            
            //Save our last rotary reading  
            LastRotary = RotaryValue;
        }             
  }
  else
  {
      Serial.println("Not connected to the cloud yet...");
  }

  Serial.println("Sleeping for 5 Seconds..." );  
  dwClient.dwSleep(5000);
  
}


void ExtractKeyValue( char *haystack, char *Key, char *ResultValue )
{
  char *KeyPtr;
  char *KeyValue;
  char *KeyValueEnd;
  int   KeyValueLen;
  
  char needleTmp[40];

   //Serial.print("ExtractKeyValue: ");Serial.println(Key);
   //Serial.print("ExtractHaystack: ");Serial.println(haystack);
              
    strcpy( needleTmp, "\"");
    strcat( needleTmp, Key);
    strcat( needleTmp, "\":\"");

    KeyPtr = strstr(haystack, needleTmp );
      
    if( KeyPtr != NULL) 
    {   
        KeyValue = KeyPtr + strlen(needleTmp);
        KeyValueEnd = KeyValue;
    
        while( *KeyValueEnd != '\"')
        {
            KeyValueEnd++;
        }
    
        KeyValueLen = KeyValueEnd - KeyValue;
        strncpy( ResultValue, KeyValue, KeyValueLen );
        *(ResultValue + KeyValueLen ) = NULL;
    }
    else
    {
        strcpy( ResultValue, "" );
    }
    
    //Serial.print("ExtractKeyValue Result: ");Serial.println(ResultValue);
}
        
// Sets the LED accordingly
void setLED( int LEDPin, char *LEDStatus )
{

    if ( strcmp(LEDStatus, "1") == NULL )
    {        
        //Serial.println("LED On!!!");          
        digitalWrite(LEDPin, HIGH);
    }
    else
    {
        //Serial.println("LED Off!!!");          
        digitalWrite(LEDPin, LOW);          
    }
   
}

// HandlesBuzzer Beep
void handleBuzzer( char *Value )
{
  int Duration = atoi( Value );

  Serial.print("in handleBuzzer: ");  Serial.println( Value );   

  digitalWrite(BUZZER_PIN, HIGH);   
  delay(Duration);
  digitalWrite(BUZZER_PIN, LOW);   
}

//TR50 JSON Sample Request/Response
#define MAILBOX_IN_PROGRESS_ACK  "{\"999-999\": { \"command\": \"mailbox.check\", \"params\": { \"autoComplete\": true, \"limit\": 0 }}}"
#define MAILBOX_REQ_RECEIVED     "{\"0000123\": { \"command\": \"mailbox.update\", \"params\": {\"id\": \"%s\", \"msg\": \"Executing...\"}}}"
#define MAILBOX_REQ_ACK          "{\"0000124\": { \"command\": \"mailbox.ack\", \"params\": {\"id\": \"%s\", \"errorCode\": \"0\", \"errorMessage\": \"OK\"}}}"

// The function is called when an MQTT message arrives
void callback(char *topic, uint8_t *payload, unsigned int length) 
{
  int i;
  char jsonText[1024];
  char Value[10];   
  char msgID[50];

  //Serial.println("Callback - MQTT Message Arrived...");
  //Serial.print("Topic: ");  Serial.println( topic ); 
  
  //Serial.print("Payload Length: ");  Serial.println( length ); 
  //Serial.print("Payload: ");  Serial.println( (char *) payload );
  
  if(strstr(topic, MQTT_NOTIFY) != NULL) 
  {  
      //Serial.println("Notification - Mailbox Activity... Check Mailbox");
      dwClient.publish("api", MAILBOX_IN_PROGRESS_ACK );    
  }
  else if( (strstr(topic, MQTT_REPLY) != NULL) )
  {
      if ((strstr((char *)payload, "999-999") != NULL)) 
      {     
        //Serial.println("Reply - Mailbox Check Response... Mailbox Request Received");   
        ExtractKeyValue( (char *) payload, "id", msgID ); 
        
        sprintf( jsonText, MAILBOX_REQ_RECEIVED, msgID );  
        dwClient.publish("api", jsonText );
  
        //Set LED accordingly                 
        ExtractKeyValue( (char *) payload, "varled", Value );                
        if ( strlen(Value) ) setLED(RED_LED, Value);

        //Handle if Buzzer
        ExtractKeyValue( (char *) payload, "duration", Value );                
        if ( strlen(Value) ) handleBuzzer(Value);        
          
        //Serial.println("Processing Complete - Send Mailbox Acknowledge...");
        sprintf( jsonText, MAILBOX_REQ_ACK, msgID );   
        dwClient.publish( "api", jsonText );  
      } 
    
  }
  
}



